# -*- coding: cp1250 -*-
import pygame, random
from function import *
from constants import *
from button import *
import ui

HUMANS = 1
COMPS = 0
class Star:
    def __init__(self, x=-1, y=-1, image = "random", ratio = "random"):
        if image == "random":
            image = random.choice(["star1.png", "star2.png", "star3.png", "star4.png", "star5.png", "star5.png", "star6.png", "star1.png"])
        self.image = load_image(image, -1, True)
        if ratio == "random":
            ratio = random.random()*STAR_RATIO
        if ratio != 1:
            size = int(self.image.get_width()*ratio), int(self.image.get_height()*ratio)
            self.image = pygame.transform.scale(self.image, size)
        self.rect = self.image.get_rect()
        if x == -1:
            x = random.randint(0,WIDTH)
        if y == -1:
            y = random.randint(0,HEIGHT)
        self.rect.center = x,y
class Planet:
    def __init__(self, x, y, star, star_mass, image = "particle1.png"):
        self.image = load_image(image, -1, True)
        self.rect = self.image.get_rect()
        self.rect.center = x,y
        self.x, self.y = float(x), float(y)
        self.starx, self.stary = star
        self.star_mass = star_mass
        self.xspeed = self.stary - self.y
        self.yspeed = self.x - self.starx
        self.velocity = (self.star_mass/(((self.xspeed)**2 + (self.yspeed)**2)**0.5))**0.5
        k = float(self.velocity / (((self.xspeed)**2 + (self.yspeed)**2)**0.5))
        self.xspeed *= k
        self.yspeed *= k
        
    def update(self):
        #self.velocity = (self.star_mass/(((self.xspeed)**2 + (self.yspeed)**2)**0.5))**0.5
        self.xspeed = self.stary - self.y
        self.yspeed = self.x - self.starx
        k = float(self.velocity / (((self.xspeed)**2 + (self.yspeed)**2)**0.5))
        self.xspeed *= k
        self.yspeed *= k
        self.x += self.xspeed
        self.y += self.yspeed
        self.rect.center = self.x, self.y
class Lane:
    def __init__(self, x, y, w, h, d = "u", speed = LANEACC): #d for direction
        self.basic_image = load_image("stream.png", -1, True)
        self.basic_image.set_alpha(100)
        self.x_acc, self.y_acc = 0, 0 # defines direction that lane is pushing
        if d == "u":
            self.y_acc = -1
            self.basic_image = pygame.transform.rotate(self.basic_image, 90)
        elif d == "r":
            self.x_acc = 1
        elif d == "d":
            self.y_acc = 1
            self.basic_image = pygame.transform.rotate(self.basic_image, 270)
        else:
            self.x_acc = -1
            self.basic_image = pygame.transform.rotate(self.basic_image, 180)
        self.image = self.basic_image.copy()
        self.particles = []
        self.rect = pygame.Rect(x, y, w, h)
        self.d = d
        self.speed = speed # 
        for p in range(PARTICLES):
            part = Particle(d)
            for i in range(5):
                self.update()
            self.particles.append(part)
    def update(self):
        self.image = self.basic_image.copy()
        for part in self.particles:
            part.update()
        if self.d == "u":
            for part in self.particles:
                self.image.blit(part.image, (part.y-part.image.get_width()/2, self.rect.h-part.x-part.image.get_height()/2))
        elif self.d == "r":
            for part in self.particles:
                self.image.blit(part.image, (part.x-part.image.get_width()/2, part.y-part.image.get_height()/2))
        elif self.d == "d":
            for part in self.particles:
                self.image.blit(part.image, (part.y-part.image.get_width()/2, part.x-part.image.get_height()/2))
        else:
            for part in self.particles:
                self.image.blit(part.image, (self.rect.w-part.x-part.image.get_width()/2, part.y-part.image.get_height()/2))
class Particle:
    def __init__(self,d):
        i = random.choice(["particle1.png", "particle2.png", "particle3.png"])
        self.d = d
        self.image = load_image(i, BLACK, True)
        if d == "u":
            self.image = pygame.transform.rotate(self.image, 90)
        elif d == "d":
            self.image = pygame.transform.rotate(self.image, 270)
        elif d == "l":
            self.image = pygame.transform.rotate(self.image, 180)
        self.x = float(random.randint(10, 80))
        self.y = float(random.randint(12, 38))
        self.basic_y = self.y
        self.xspeed = 0.0
        self.yspeed = (random.random()-0.5)*PART_WAVE_RANGE
    def update(self):
        self.x += self.xspeed
        self.xspeed += PARTACC
        if self.xspeed > PARTSPEED:
            self.xspeed = PARTSPEED
        if self.x >= 180:
            self.__init__(self.d)
        self.y += self.yspeed
        self.yspeed += (self.basic_y - self.y) * PART_WAVE_K
class Atractor:
    def __init__(self, x,y, power):
        self.rect = pygame.Rect(x,y, 1, 1)
        self.power = power
class Player:
    def __init__(self, axis, comp = False):
        #self.basic_image = load_image("ufo1.png", -1, True)
        self.basic_image = load_random_image()
        self.image = self.basic_image
        self.rect = self.image.get_rect()
        self.rect.topleft = axis
        self.angle = 0.0
        self.rotate = float(5)
        self.xspeed = 0.0
        self.yspeed = 0.0
        self.comp = comp
        self.xforceacc = 0.0
        self.yforceacc = 0.0
        for i in range(random.randint(0,20)):
            self.wave()
        
    def move(self, u, l, d, r, xforce=0, yforce=0):
        self.wave()
        self.xforceacc += xforce
        if self.xforceacc > LANESPEED:
            self.xforceacc = LANESPEED
            self.xforce = 0
        elif self.xforceacc < -LANESPEED:
            self.xforceacc = -LANESPEED
            self.xforce = 0
        elif xforce == 0:
            self.xforceacc = 0
        self.yforceacc += yforce
        if self.yforceacc > LANESPEED:
            self.yforceacc = LANESPEED
            self.yforce = 0
        elif self.yforceacc < -LANESPEED:
            self.yforceacc = -LANESPEED
            self.yforce = 0
        elif yforce == 0:
            self.yforceacc = 0
        if l:
            self.xspeed -= ACC
        if r:
            self.xspeed += ACC
        if d:
            self.yspeed += ACC
        if u:
            self.yspeed -= ACC
        self.xspeed += xforce
        self.yspeed += yforce
        if ((self.xspeed-self.xforceacc)**2 + (self.yspeed-self.yforceacc)**2)**0.5 > MAXSPEED:
            k = float(MAXSPEED / (((self.xspeed-self.xforceacc)**2 + (self.yspeed-self.yforceacc)**2)**0.5))
            self.xspeed *= k
            self.yspeed *= k
        self.xspeed *= DRAG
        self.yspeed *= DRAG
        self.rect.x += self.xspeed
        if self.rect.x < 0:
            self.rect.x = 0
            self.xspeed = -self.xspeed
        if self.rect.right > WIDTH:
            self.rect.right = WIDTH-1
            self.xspeed = -self.xspeed
        self.rect.y += self.yspeed
        if self.rect.y < 0:
            self.rect.y = 0
            self.yspeed = -self.yspeed
        if self.rect.bottom > HEIGHT:
            self.rect.bottom = HEIGHT-1
            self.yspeed = -self.yspeed
    def wave(self):
        self.angle += self.rotate
        self.rotate += -self.angle*ANGLE_ACC_K
        """number = 190
        self.rotate *= (number+random.random())/number+1"""
        new_image = pygame.transform.rotate(self.basic_image, self.angle)
        new_rect = new_image.get_rect()
        new_rect.center = self.rect.center
        self.rect = new_rect
        self.image = new_image
class Planees:
    def __init__(self, humans, comps):
        pygame.mouse.set_visible(False)
        HUMANS = humans
        COMPS = comps
        self.screen = pygame.display.set_mode((WIDTH,HEIGHT))
        self.screen.fill(BLACK)
        stars = []
        the_star = Star(400,300,"star.png", 1)
        for i in range(STARS):
            stars.append(Star())
        for star in stars:
            self.screen.blit(star.image, star.rect)
        self.screen.blit(the_star.image, the_star.rect)
        self.bg = self.screen.copy()
        planets = []
        star_mass = 200
        offsetx = random.randint(-25, 25)
        offsety = random.randint(-25, 25)
        planets.append(Planet(350+offsetx,250+offsety, (400,300), star_mass, "mars.png"))
        planets.append(Planet(400+offsetx,200+offsety, (400,300), star_mass, "venus.png"))
        planets.append(Planet(500+offsetx,100+offsety, (400,300), star_mass, "jupiter.png"))
        planets.append(Planet(520+offsetx,500+offsety, (400,300), star_mass, "uran.png"))
        planets.append(Planet(200+offsetx,500+offsety, (400,300), star_mass, "neptune.png"))
        running = True
        c = pygame.time.Clock()
        lanes = []
        atractors = []
        lanes = []
        bwl = BORDER_WITHOUT_LANES # just renaming for shorter typing
        # for:
        for i in range(LANES):
        # vytvo nhodn data pro psy (x, y, w, h, d, ppadn i speed)
            d = random.choice("uldr")
            if d in "ud":
                x = random.randint(bwl, WIDTH - (bwl + LANEWIDTH))
                y = random.randint(bwl, HEIGHT - (bwl + LANELENGHT))
                w = LANEWIDTH
                h = LANELENGHT
            else:
                x = random.randint(bwl, WIDTH - (bwl + LANELENGHT))
                y = random.randint(bwl, HEIGHT - (bwl + LANEWIDTH))
                w = LANELENGHT
                h = LANEWIDTH
        # ulo psy s daty do lanes[]
            lanes.append(Lane(x,y,w,h,d))
   
        chars = []
        uis = []
        for i in range(HUMANS + COMPS):
            chars.append(Player((random.randint(0,3)*WIDTH/4+WIDTH/8,random.randint(0,2)*HEIGHT/3+HEIGHT/6), False))
        for i in range(COMPS):
            uis.append(ui.Ui(chars[i].rect.center, (WIDTH/2, HEIGHT/2)))
        if True:
            tendencies = []
            for i in range(COMPS):
                tendencies.append([])
            for i in range(HUMANS + COMPS):
                if i < COMPS:
                    chars[i].comp = True
                    total_tends = 0
                    for o in range(HUMANS + COMPS):
                        if o != i:
                            tendency = float((random.randint(1,1)-0.4)*5.0) ## - total_tends)
                            total_tends += tendency
                            tendencies[i].append([o, tendency])
                            if o < COMPS and False:
                                tendencies[o].append([o, -tendency])


        time = pygame.time.get_ticks() / 100
        frames = FRAMES_B4_START
        while running:
            c.tick(20)
            #self.screen.fill(BLACK)
            self.screen.blit(self.bg, (0,0))
            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    running = False
                if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
                    running = False
            key = pygame.key.get_pressed()
            # generuj seznam pro ui
            
            for i in range(HUMANS + COMPS):
                if frames > 0:
                    frames -= 1
                elif i < COMPS:
                    tends = []
                    for tendence in tendencies[i]:
                        tends.append([chars[tendence[0]].rect.center, tendence[1]])
                    u, l, d, r = uis[i].move(chars[i].rect.center, tends)
                    xforce, yforce = 0, 0
                    for la in lanes:
                        if la.rect.colliderect(chars[i].rect):
                            xforce += la.x_acc * la.speed
                            yforce += la.y_acc * la.speed
                    chars[i].xspeed += xforce
                    chars[i].yspeed += yforce
                    chars[i].move(u, l, d, r, xforce, yforce)
                    """chars[i].xspeed -= xforce
                    chars[i].yspeed -= yforce"""
                else:
                    hi = i - COMPS
                    xforce, yforce = 0, 0
                    for l in lanes:
                        if l.rect.collidepoint(chars[i].rect.center):
                            xforce += l.x_acc * l.speed
                            yforce += l.y_acc * l.speed
                    chars[i].xspeed += xforce
                    chars[i].yspeed += yforce
                    chars[i].move(key[KEYS[hi][0]], key[KEYS[hi][1]], key[KEYS[hi][2]], key[KEYS[hi][3]], xforce, yforce)
                    """chars[i].xspeed -= xforce
                    chars[i].yspeed -= yforce"""
                            
            for i in range(len(chars)):
                for o in range(len(chars)):
                    if o != i:                        
                        if chars[o].rect.colliderect(chars[i].rect):
                            print (pygame.time.get_ticks() / 100 - time) / 10., 's'
                            running = False
    # --- zatek vykreslovn ---
            
            for p in planets:
                p.update()
                self.screen.blit(p.image, p.rect)
            
            for l in lanes:
                #pygame.draw.rect(self.screen, (127, 127, 127), l.rect)
                l.update()
                self.screen.blit(l.image, l.rect.topleft)
            for i in chars:
                self.screen.blit(i.image, i.rect.topleft)
            #self.screen.blit(char2.image, char2.rect.topleft)
            pygame.display.flip()
            for i in chars:
                self.screen.blit(self.bg.subsurface(i.rect), i.rect.topleft)
            #self.screen.blit(self.bg.subsurface(char2.rect), char2.rect.topleft)"""
        pygame.mouse.set_visible(True)
def start_1_1():
    HUMANS = 1
    COMPS = 1
    Planees(1,1)

def start_2_1():
    HUMANS = 2
    COMPS = 1
    Planees(2,1)

def start_2_0():
    HUMANS = 2
    COMPS = 0
    Planees(2,0)


def end():
    global running
    running = False
    
def main():
    global running
    pygame.init()
    #player.play()
    screen = pygame.display.set_mode((800,600))
    screen.fill(BLACK)
    pygame.display.set_caption('SW by W for PW')
    pygame.display.flip()

    buttons = []
    buttons.append(Button((50,150), action=start_1_1, text="human VS. comp", fontsize=40, background=BLACK, colour=(0,200,8), oncolour=(50,255,50)))
    buttons.append(Button((70,230), action=start_2_0, text="human VS. human", fontsize=40, background=BLACK, colour=(0,200,8), oncolour=(50,255,50)))
    buttons.append(Button((90,310), action=start_2_1, text="2 humans VS. 1 comp", fontsize=40, background=BLACK, colour=(0,200,8), oncolour=(50,255,50)))
    buttons.append(Button((110,390), action=end, text="QUIT", fontsize=40, background=BLACK, colour=(169,0,0)))
    #buttons.append(Button((250,500), action=toggle_sound, text="SOUND", width=300, fontsize=30, background=(255,255,255)))
    
    
    stars = []
    the_star = Star(400,300,"star.png", 1)
    for i in range(STARS):
        stars.append(Star())
    for star in stars:
        screen.blit(star.image, star.rect)
    screen.blit(the_star.image, the_star.rect)
    background = screen.copy()
    planets = []
    star_mass = 120
    planets.append(Planet(350,350, (400,300), star_mass, "mars.png"))
    planets.append(Planet(300,300, (400,300), star_mass, "venus.png"))
    planets.append(Planet(200,200, (400,300), star_mass, "jupiter.png"))
    planets.append(Planet(600,180, (400,300), star_mass, "uran.png"))
    planets.append(Planet(600,500, (400,300), star_mass, "neptune.png"))
    running = True
    c = pygame.time.Clock()
    
    while running:
        c.tick(20)
        screen.blit(background, (0,0))
        for e in pygame.event.get():
            if e.type == pygame.QUIT:
                running = False
            if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
                running = False
            if e.type == pygame.MOUSEBUTTONDOWN and e.button == 1:
                for b in buttons:
                    if b.rect.collidepoint(e.pos):
                        b.pressed()
            if e.type == pygame.MOUSEBUTTONUP and e.button == 1:
                for b in buttons:
                    b.mouse_release(b.rect.collidepoint(e.pos))

        for p in planets:
            p.update()
            screen.blit(p.image, p.rect)
        for b in buttons:
            screen.blit(b.image, b.rect)
        


        pygame.display.flip()
    
if __name__ == "__main__":
    pygame.init()
    main()
    pygame.quit()
